//*****************************************************************************
//
// ektm4c129_epiflash_bootloader.c - The main file to prototype EPI Flash Functions.
//
// Copyright (c) 2013-2014 Texas Instruments Incorporated.  All rights reserved.
// Software License Agreement
//
// Texas Instruments (TI) is supplying this software for use solely and
// exclusively on TI's microcontroller products. The software is owned by
// TI and/or its suppliers, and is protected under applicable copyright
// laws. You may not combine this software with "viral" open-source
// software in order to form a larger program.
//
// THIS SOFTWARE IS PROVIDED "AS IS" AND WITH ALL FAULTS.
// NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT
// NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. TI SHALL NOT, UNDER ANY
// CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
// DAMAGES, FOR ANY REASON WHATSOEVER.
//
// This is part of revision 2.1.0.12573 of the EK-TM4C1294XL Firmware Package.
//
//*****************************************************************************
#include <stdint.h>
#include <stdbool.h>
#include "inc/hw_epi.h"
#include "inc/hw_gpio.h"
#include "inc/hw_memmap.h"
#include "inc/hw_nvic.h"
#include "inc/hw_sysctl.h"
#include "inc/hw_types.h"
#include "inc/hw_uart.h"
#include "bl_config.h"
#include "ektm4c129_epiflash_bootloader.h"

//*****************************************************************************
//
// Defines for GPIO Pins for EPI0, USER_SWx and LED
//
//*****************************************************************************
#define EPI_PORTA_PINS  0xC0
#define EPI_PORTB_PINS  0x0C
#define EPI_PORTC_PINS  0xF0
#define EPI_PORTG_PINS  0x03
#define EPI_PORTH_PINS  0x0F
#define EPI_PORTK_PINS  0xF0
#define EPI_PORTL_PINS  0x3F
#define EPI_PORTM_PINS  0x0F
#define EPI_PORTN_PINS  0x30
#define EPI_PORTP_PINS  0x0C
#define EPI_PORTQ_PINS  0x0F

#define EPI_PORTA_PCTL  0xFF000000
#define EPI_PORTB_PCTL  0x0000FF00
#define EPI_PORTC_PCTL  0xFFFF0000
#define EPI_PORTG_PCTL  0x000000FF
#define EPI_PORTH_PCTL  0x0000FFFF
#define EPI_PORTK_PCTL  0xFFFF0000
#define EPI_PORTL_PCTL  0x00FFFFFF
#define EPI_PORTM_PCTL  0x0000FFFF
#define EPI_PORTN_PCTL  0x00FF0000
#define EPI_PORTP_PCTL  0x0000FF00
#define EPI_PORTQ_PCTL  0x0000FFFF

#define LED_DIAG0_PINS  0x01
#define LED_DIAG1_PINS  0x02

#define USER_SW1_PINS   0x01
#define USER_SW2_PINS   0x02

//*****************************************************************************
//
// Defines for Flash and SRAM Address
//
//*****************************************************************************
#define FLASH_APP_START_ADDRESS 0x60000000
#define FLASH_SECTOR_SIZE       0x10000
#define SRAM_APP_START_ADDRESS  0xA0000000

//*****************************************************************************
//
// A prototype for the function (in the startup code) for a predictable length
// delay.
//
//*****************************************************************************
extern void Delay(uint32_t ui32Count);

//*****************************************************************************
//
// Global Variable to hold the Transfer Address and Size
//
//*****************************************************************************
extern uint32_t g_ui32TransferAddress;
extern uint32_t g_ui32TransferSize;

//*****************************************************************************
//
//! Configures the IO's of LED Module for Diagnostics
//!
//! \return None.
//
//*****************************************************************************
void
ConfigureLEDIOs(void)
{
	//
	// Enable Diagnostic Port N for LED Blinking
	//
	HWREG(SYSCTL_RCGCGPIO) |= SYSCTL_RCGCGPIO_R12;

	//
	// Wait for the Clocks to be enabled for Port N module
	//
	while((HWREG(SYSCTL_PRGPIO) & SYSCTL_PRGPIO_R12) != SYSCTL_PRGPIO_R12);

	//
	// Configure PN0 and PN1 LED Pins for Diagnosis Message
	// and drive them to 0.
	//
	HWREG(GPIO_PORTN_BASE + GPIO_O_AFSEL)     &= ~(LED_DIAG0_PINS | LED_DIAG1_PINS);
	HWREG(GPIO_PORTN_BASE + GPIO_O_DIR)       |= (LED_DIAG0_PINS | LED_DIAG1_PINS);
	HWREG(GPIO_PORTN_BASE + GPIO_O_DEN)       |= (LED_DIAG0_PINS | LED_DIAG1_PINS);
	HWREG(GPIO_PORTN_BASE + ((LED_DIAG0_PINS | LED_DIAG1_PINS) << 2))
						= 0x0;
}

//*****************************************************************************
//
//! Configures the IO's of User Switch for Boot Mode Selection
//!
//! \return None.
//
//*****************************************************************************
void
ConfigureUserSwitchIOs(void)
{
	//
	// Enable Diagnostic Port J for User Switch
	//
	HWREG(SYSCTL_RCGCGPIO) |= SYSCTL_RCGCGPIO_R8;

	//
	// Wait for the Clocks to be enabled for Port J module
	//
	while((HWREG(SYSCTL_PRGPIO) & SYSCTL_PRGPIO_R8) != SYSCTL_PRGPIO_R8);

	//
	// Configure PJ0 for User Switch as Input with Pull Up
	//
	HWREG(GPIO_PORTJ_AHB_BASE + GPIO_O_AFSEL)     &= ~(USER_SW1_PINS);
	HWREG(GPIO_PORTJ_AHB_BASE + GPIO_O_DIR)       &= ~(USER_SW1_PINS);
	HWREG(GPIO_PORTJ_AHB_BASE + GPIO_O_PUR)       |= (USER_SW1_PINS);
	HWREG(GPIO_PORTJ_AHB_BASE + GPIO_O_DEN)       |= (USER_SW1_PINS);

	//
	// Configure PJ1 for User Switch as Input with Pull Up
	//
	HWREG(GPIO_PORTJ_AHB_BASE + GPIO_O_AFSEL)     &= ~(USER_SW2_PINS);
	HWREG(GPIO_PORTJ_AHB_BASE + GPIO_O_DIR)       &= ~(USER_SW2_PINS);
	HWREG(GPIO_PORTJ_AHB_BASE + GPIO_O_PUR)       |= (USER_SW2_PINS);
	HWREG(GPIO_PORTJ_AHB_BASE + GPIO_O_DEN)       |= (USER_SW2_PINS);

}

//*****************************************************************************
//
//! Configures the IO's of EPI Module for Parallel Flash and SRAM
//!
//! \return None.
//
//*****************************************************************************
void
ConfigureEPIIOs(void)
{
	//
	// Enable GPIO Port A, B, C, G, H, K, L, M, N, P and Q for EPI
	//
	HWREG(SYSCTL_RCGCGPIO) |= (SYSCTL_RCGCGPIO_R0 | SYSCTL_RCGCGPIO_R1 |
			SYSCTL_RCGCGPIO_R2 | SYSCTL_RCGCGPIO_R6 | SYSCTL_RCGCGPIO_R7 |
			SYSCTL_RCGCGPIO_R9 | SYSCTL_RCGCGPIO_R10 | SYSCTL_RCGCGPIO_R11 |
			SYSCTL_RCGCGPIO_R12 | SYSCTL_RCGCGPIO_R13 | SYSCTL_RCGCGPIO_R14);

	//
	// Wait for the Clocks to be enabled for Port A, B, C, G, H, K, L, M,
	// N, P and Q
	//
	while((HWREG(SYSCTL_PRGPIO) & (SYSCTL_PRGPIO_R0 | SYSCTL_PRGPIO_R1 |
			SYSCTL_PRGPIO_R2 | SYSCTL_PRGPIO_R6 | SYSCTL_PRGPIO_R7 |
			SYSCTL_PRGPIO_R9 | SYSCTL_PRGPIO_R10 | SYSCTL_PRGPIO_R11 |
			SYSCTL_PRGPIO_R12 | SYSCTL_PRGPIO_R13 | SYSCTL_PRGPIO_R14)) !=
					(SYSCTL_PRGPIO_R0 | SYSCTL_PRGPIO_R1 |
					SYSCTL_PRGPIO_R2 | SYSCTL_PRGPIO_R6 | SYSCTL_PRGPIO_R7 |
					SYSCTL_PRGPIO_R9 | SYSCTL_PRGPIO_R10 | SYSCTL_PRGPIO_R11 |
					SYSCTL_PRGPIO_R12 | SYSCTL_PRGPIO_R13 | SYSCTL_PRGPIO_R14));

	//
	// Configure EPI IO Pins
	//
	HWREG(GPIO_PORTA_AHB_BASE + GPIO_O_PCTL)  |= EPI_PORTA_PCTL;
	HWREG(GPIO_PORTA_AHB_BASE + GPIO_O_DEN)   |= EPI_PORTA_PINS;
	HWREG(GPIO_PORTA_AHB_BASE + GPIO_O_DR8R)  |= EPI_PORTA_PINS;
	HWREG(GPIO_PORTA_AHB_BASE + GPIO_O_AFSEL) |= EPI_PORTA_PINS;

	HWREG(GPIO_PORTB_AHB_BASE + GPIO_O_PCTL)  |= EPI_PORTB_PCTL;
	HWREG(GPIO_PORTB_AHB_BASE + GPIO_O_DEN)   |= EPI_PORTB_PINS;
	HWREG(GPIO_PORTB_AHB_BASE + GPIO_O_DR8R)  |= EPI_PORTB_PINS;
	HWREG(GPIO_PORTB_AHB_BASE + GPIO_O_AFSEL) |= EPI_PORTB_PINS;

	HWREG(GPIO_PORTC_AHB_BASE + GPIO_O_PCTL)  |= EPI_PORTC_PCTL;
	HWREG(GPIO_PORTC_AHB_BASE + GPIO_O_DEN)   |= EPI_PORTC_PINS;
	HWREG(GPIO_PORTC_AHB_BASE + GPIO_O_DR8R)  |= EPI_PORTC_PINS;
	HWREG(GPIO_PORTC_AHB_BASE + GPIO_O_AFSEL) |= EPI_PORTC_PINS;

	HWREG(GPIO_PORTG_AHB_BASE + GPIO_O_PCTL)  |= EPI_PORTG_PCTL;
	HWREG(GPIO_PORTG_AHB_BASE + GPIO_O_DEN)   |= EPI_PORTG_PINS;
	HWREG(GPIO_PORTG_AHB_BASE + GPIO_O_DR8R)  |= EPI_PORTG_PINS;
	HWREG(GPIO_PORTG_AHB_BASE + GPIO_O_AFSEL) |= EPI_PORTG_PINS;

	HWREG(GPIO_PORTH_AHB_BASE + GPIO_O_PCTL)  |= EPI_PORTH_PCTL;
	HWREG(GPIO_PORTH_AHB_BASE + GPIO_O_DEN)   |= EPI_PORTH_PINS;
	HWREG(GPIO_PORTH_AHB_BASE + GPIO_O_DR8R)  |= EPI_PORTH_PINS;
	HWREG(GPIO_PORTH_AHB_BASE + GPIO_O_AFSEL) |= EPI_PORTH_PINS;

	HWREG(GPIO_PORTK_BASE + GPIO_O_PCTL)  |= EPI_PORTK_PCTL;
	HWREG(GPIO_PORTK_BASE + GPIO_O_DEN)   |= EPI_PORTK_PINS;
	HWREG(GPIO_PORTK_BASE + GPIO_O_DR8R)  |= EPI_PORTK_PINS;
	HWREG(GPIO_PORTK_BASE + GPIO_O_AFSEL) |= EPI_PORTK_PINS;

	HWREG(GPIO_PORTL_BASE + GPIO_O_PCTL)  |= EPI_PORTL_PCTL;
	HWREG(GPIO_PORTL_BASE + GPIO_O_DEN)   |= EPI_PORTL_PINS;
	HWREG(GPIO_PORTL_BASE + GPIO_O_DR8R)  |= EPI_PORTL_PINS;
	HWREG(GPIO_PORTL_BASE + GPIO_O_AFSEL) |= EPI_PORTL_PINS;

	HWREG(GPIO_PORTM_BASE + GPIO_O_PCTL)  |= EPI_PORTM_PCTL;
	HWREG(GPIO_PORTM_BASE + GPIO_O_DEN)   |= EPI_PORTM_PINS;
	HWREG(GPIO_PORTM_BASE + GPIO_O_DR8R)  |= EPI_PORTM_PINS;
	HWREG(GPIO_PORTM_BASE + GPIO_O_AFSEL) |= EPI_PORTM_PINS;

	HWREG(GPIO_PORTN_BASE + GPIO_O_PCTL)  |= EPI_PORTN_PCTL;
	HWREG(GPIO_PORTN_BASE + GPIO_O_DEN)   |= EPI_PORTN_PINS;
	HWREG(GPIO_PORTN_BASE + GPIO_O_DR8R)  |= EPI_PORTN_PINS;
	HWREG(GPIO_PORTN_BASE + GPIO_O_AFSEL) |= EPI_PORTN_PINS;

	HWREG(GPIO_PORTP_BASE + GPIO_O_PCTL)  |= EPI_PORTP_PCTL;
	HWREG(GPIO_PORTP_BASE + GPIO_O_DEN)   |= EPI_PORTP_PINS;
	HWREG(GPIO_PORTP_BASE + GPIO_O_DR8R)  |= EPI_PORTP_PINS;
	HWREG(GPIO_PORTP_BASE + GPIO_O_AFSEL) |= EPI_PORTP_PINS;

	HWREG(GPIO_PORTQ_BASE + GPIO_O_PCTL)  |= EPI_PORTQ_PCTL;
	HWREG(GPIO_PORTQ_BASE + GPIO_O_DEN)   |= EPI_PORTQ_PINS;
	HWREG(GPIO_PORTQ_BASE + GPIO_O_DR8R)  |= EPI_PORTQ_PINS;
	HWREG(GPIO_PORTQ_BASE + GPIO_O_AFSEL) |= EPI_PORTQ_PINS;
}

//*****************************************************************************
//
//! Configures the EPI Interface for SDRAM and check if it accessible
//!
//! \return 0 for Failure of EPI SDRAM Init or 1 for Success.
//
//*****************************************************************************
uint32_t
ConfigureEPIInterface(void)
{
	uint32_t ui32ReadID[4];

	//
	// Disable and Reset the EPI0 Module at every start
	//
	HWREG(SYSCTL_RCGCEPI)  &= ~(SYSCTL_RCGCEPI_R0);
	HWREG(SYSCTL_SREPI)    |= (SYSCTL_RCGCEPI_R0);
	Delay(16);
	HWREG(SYSCTL_SREPI)    &= ~(SYSCTL_RCGCEPI_R0);

	//
	// Enable Clock to EPI0 Module
	//
	HWREG(SYSCTL_RCGCEPI)  |= (SYSCTL_RCGCEPI_R0);

	//
	// Wait for the Clocks to be enabled for EPI0 module
	// as it is the last of the modules to be enabled
	//
	while((HWREG(SYSCTL_PREPI) & SYSCTL_PREPI_R0) != SYSCTL_PREPI_R0);

	//
	// Set the Divider for Clock Same as the System Clock
	//
    HWREG(EPI0_BASE + EPI_O_BAUD) = 0x1;

    //
    // Write the mode word to the register for HB16 Mode
    //
    HWREG(EPI0_BASE + EPI_O_CFG) = (EPI_CFG_BLKEN | EPI_CFG_MODE_HB16);

    //
    // Write the HB16 configuration register for CS0
    //
    HWREG(EPI0_BASE + EPI_O_HB16CFG) = (EPI_HB16CFG_ALEHIGH | EPI_HB16CFG_MAXWAIT_M | EPI_HB16CFG_WRWS_8 |
    		EPI_HB16CFG_RDWS_8 | EPI_HB16CFG_BSEL);

    //
    // Write the HB16 configuration register for CS1
    //
    HWREG(EPI0_BASE + EPI_O_HB16CFG2) = (EPI_HB8CFG2_CSCFG_ADCS | EPI_HB16CFG_ALEHIGH | EPI_HB16CFG_WRWS_2 |
    		EPI_HB16CFG_WRWS_2 | EPI_HB16CFG_BSEL);

    //
    // Configure the Timing Parameter for the FLASH
    // and SRAM
    //
    HWREG(EPI0_BASE + EPI_O_HB16TIME)  = (EPI_HB16TIME_WRWSM | EPI_HB16TIME_RDWSM);
    HWREG(EPI0_BASE + EPI_O_HB16TIME2) = (EPI_HB16TIME2_WRWSM | EPI_HB16TIME2_RDWSM);

    //
    // Configure the Address Space Size for FLASH on CS0 and SRAM
    // on CS1
    //

    HWREG(EPI0_BASE + EPI_O_ADDRMAP) = (EPI_ADDRMAP_EPSZ_256MB | EPI_ADDRMAP_EPADR_A000
										| EPI_ADDRMAP_ERSZ_256MB | EPI_ADDRMAP_ERADR_6000);

    //
    // Check SRAM Location for Read and Write Access
    //
    HWREGB(SRAM_APP_START_ADDRESS)       = 0xDD;
    HWREGB(SRAM_APP_START_ADDRESS + 0x1) = 0x66;
    HWREGB(SRAM_APP_START_ADDRESS + 0x2) = 0xCC;
    HWREGB(SRAM_APP_START_ADDRESS + 0x3) = 0x44;
    HWREG(SRAM_APP_START_ADDRESS + 0x4) = ~(HWREG(SRAM_APP_START_ADDRESS));

    if((HWREG(SRAM_APP_START_ADDRESS) != 0x44CC66DD) || (HWREG(SRAM_APP_START_ADDRESS+0x4) != 0xBB339922))
    {
    	return(2);
    }

    //
    // Check FLASH ID register
    //
    HWREGH(FLASH_APP_START_ADDRESS + 0xAAA) = 0x00AA;
    HWREGH(FLASH_APP_START_ADDRESS + 0x554) = 0x0055;
    HWREGH(FLASH_APP_START_ADDRESS + 0xAAA) = 0x0090;

    Delay(10);

    ui32ReadID[0] = HWREGH(FLASH_APP_START_ADDRESS + 0x0);
    ui32ReadID[1] = HWREGH(FLASH_APP_START_ADDRESS + 0x2);
    ui32ReadID[2] = HWREGH(FLASH_APP_START_ADDRESS + 0x1C);
    ui32ReadID[3] = HWREGH(FLASH_APP_START_ADDRESS + 0x1E);

    HWREGH(FLASH_APP_START_ADDRESS + 0x0) = 0x00F0;

    if(ui32ReadID[0] != 0xC2)
    {
    	return(1);
    }
    else
    {
        return(0);
    }
}

//*****************************************************************************
//
//! Reads the User Switch and returns the value
//!
//! \return Switch value of 0 (pressed) or 1 (unpressed).
//
//*****************************************************************************
uint32_t
ReadUserSwitch(uint32_t ui32Switch)
{
	return(HWREG(GPIO_PORTJ_AHB_BASE+(ui32Switch << 2)));
}

//*****************************************************************************
//
//! EPI Flash Read Function.
//!
//! \return the ui16NumberOfReadBytes bytes of read data in array pointed by
//! *ui8ReadData
//
//*****************************************************************************
void
EPIReadFunction(uint32_t ui32Address, uint8_t *ui8ReadData, uint16_t ui16NumberOfReadBytes)
{
	uint32_t ui32Count;

	for(ui32Count = 0; ui32Count < ui16NumberOfReadBytes; ui32Count++)
	{
		ui8ReadData[ui32Count] = HWREGB(FLASH_APP_START_ADDRESS + ui32Address + ui32Count);
	}

}

//*****************************************************************************
//
//! EPI Sector Erase Function.
//!
//! \return None
//
//*****************************************************************************
void
EPIEraseFunction(uint32_t ui32Sector)
{
	uint8_t  ui8ReadData;

	HWREGB(FLASH_APP_START_ADDRESS + 0xAAA) = 0x00AA;
    HWREGB(FLASH_APP_START_ADDRESS + 0x554) = 0x0055;

    HWREGB(FLASH_APP_START_ADDRESS + 0xAAA) = 0x0080;
    HWREGB(FLASH_APP_START_ADDRESS + 0xAAA) = 0x00AA;
    HWREGB(FLASH_APP_START_ADDRESS + 0x554) = 0x0055;

    HWREGB(FLASH_APP_START_ADDRESS + (FLASH_SECTOR_SIZE * ui32Sector)) = 0x0030;

    ui8ReadData = 0;
    while((ui8ReadData & 0xFF) != 0xFF){
    	ui8ReadData = HWREGB(FLASH_APP_START_ADDRESS + (FLASH_SECTOR_SIZE * ui32Sector));
    }

    HWREGH(FLASH_APP_START_ADDRESS + 0x0) = 0x00F0;
}

//*****************************************************************************
//
//! EPI Parallel Flash Program Function.
//!
//! \return None
//
//*****************************************************************************
void
EPIProgramFunction(uint32_t ui32Address, uint8_t *pucSrcData, uint32_t ui32Length)
{
	uint32_t ui32Loop;

    HWREGH(FLASH_APP_START_ADDRESS + 0xAAA) = 0x00AA;
    HWREGH(FLASH_APP_START_ADDRESS + 0x554) = 0x0055;

    HWREGH(FLASH_APP_START_ADDRESS + ui32Address)   = 0x0025;
    HWREGH(FLASH_APP_START_ADDRESS + ui32Address)   = ((ui32Length/2) - 1);

    for(ui32Loop=0 ; ui32Loop < (ui32Length/2) ; ui32Loop++)
    {
    	HWREGH(FLASH_APP_START_ADDRESS + ui32Address + (ui32Loop*0x2)) =
    			((pucSrcData[(ui32Loop*0x2)+0x1] << 8) | pucSrcData[(ui32Loop*0x2)]);
    }

	Delay(10);
    HWREGH(FLASH_APP_START_ADDRESS + ui32Address) = 0x0029;

    Delay(1000);
}

//*****************************************************************************
//
//! Reads the EPU Flash Memory Sector 0 to check if there is an image
//| pointer. An Image pointer is qualifed as a 2 word filed
//! Word-0: upper 16-bits for Sector Start, one reserved byte and
//!         lower byte for valid image SSSSRRVV
//!         If VV == 0xFF then there is no image
//!         If VV == 0xFE then the image is a partial image at copy
//!         If VV == 0xFC then the image is a valid Image
//! Word-1: Size of Image in Bytes
//!
//! \return false if no valid image found. If valid image found then return
//! the sector number and ImageSize.
//
//*****************************************************************************
bool
GetEPIFirmWareImagePointer(uint16_t *ui16SectorNumber, uint32_t *ui32ImageSize)
{
	uint8_t ui8ImagePointer[8];
	uint32_t ui32PointerAddress;

	//
	// Get the address of the Image Pointer Table
	//
	ui32PointerAddress = (FLASH_SECTOR_SIZE * 0);

	//
	// Read 2 Words of Pointer Header to locate the image
	//
	EPIReadFunction(ui32PointerAddress, &ui8ImagePointer[0], 8);

	if(ui8ImagePointer[0] == 0xFF)
	{
		//
		// Return with no image found
		//
		return false;
	}
	else if(ui8ImagePointer[0] == 0xFC)

	{
		//
		// Return the Sector Number and Size of Imahe
		//
		*ui16SectorNumber = ui8ImagePointer[2] | (ui8ImagePointer[3] << 8);
		*ui32ImageSize    = ui8ImagePointer[4] | (ui8ImagePointer[5] << 8) | (ui8ImagePointer[6] << 16) | (ui8ImagePointer[7] << 24);
		return true;
	}
	else
	{
		//
		// Return with no image found and TODO: erase Serial
		// Flash
		//
		return false;
	}

}

//*****************************************************************************
//
// This function is used to call the user application.  It will set the NVIC
// to point at the user app's vector table, load up the user app's stack
// pointer, and then jump to the application.
//
// This function must be programmed in assembly since it needs to directly
// manipulate the value in the stack pointer, and because it needs to perform
// a direct branch to the user app and not a function call (bl).
//
//*****************************************************************************
#if defined(codered) || defined(gcc) || defined(sourcerygxx)
void __attribute__((naked))
CallApplication(uint_fast32_t ui32StartAddr)
{
    //
    // Set the vector table to the beginning of the app in flash.
    //
    HWREG(NVIC_VTABLE) = ui32StartAddr;

    //
    // Load the stack pointer from the application's vector table.
    //
    __asm("    ldr     r1, [r0]\n"
          "    mov     sp, r1");

    //
    // Load the initial PC from the application's vector table and branch to
    // the application's entry point.
    //
    __asm("    ldr     r0, [r0, #4]\n"
          "    bx      r0\n");
}
#elif defined(ewarm)
void
CallApplication(uint_fast32_t ui32StartAddr)
{
    //
    // Set the vector table to the beginning of the app in flash.
    //
    HWREG(NVIC_VTABLE) = ui32StartAddr;

    //
    // Load the stack pointer from the application's vector table.
    //
    __asm("    ldr     r1, [r0]\n"
          "    mov     sp, r1");

    //
    // Load the initial PC from the application's vector table and branch to
    // the application's entry point.
    //
    __asm("    ldr     r0, [r0, #4]\n"
          "    bx      r0\n");
}
#elif defined(rvmdk) || defined(__ARMCC_VERSION)
__asm void
CallApplication(uint_fast32_t ui32StartAddr)
{
    //
    // Set the vector table address to the beginning of the application.
    //
    ldr     r1, =0xe000ed08
    str     r0, [r1]

    //
    // Load the stack pointer from the application's vector table.
    //
    ldr     r1, [r0]
    mov     sp, r1

    //
    // Load the initial PC from the application's vector table and branch to
    // the application's entry point.
    //
    ldr     r0, [r0, #4]
    bx      r0
}
#elif defined(ccs)
void
CallApplication(uint_fast32_t ui32StartAddr)
{
    //
    // Set the vector table to the beginning of the app in flash.
    //
    HWREG(NVIC_VTABLE) = ui32StartAddr;

#if 0
    __asm("    movw r0, #0x0000\n"
          "    movt r0, #0x6000\n"
          "    ldr  sp, [r0]\n"
          "    ldr  r0, [r0, #4]\n"
          "    bx   r0");
#endif
    //
    // Load the stack pointer from the application's vector table.
    // Load the initial PC from the application's vector table and branch to
    // the application's entry point.
    //
    __asm("    ldr     sp, [r0]\n"
          "    ldr     r0, [r0, #4]\n"
          "    bx      r0\n");
}
#else
#error Undefined compiler!
#endif

//*****************************************************************************
//
//! Configures the EPI Interface for Parallel Flash and SRAM
//!
//! \return None.
//
//*****************************************************************************
void EPIFlashInitFunction(void)
{
	uint16_t ui16StartSectorNumber;
	uint32_t ui32ImageSizeInBytes;
	uint32_t ui32Status;
	bool bStatus;

	//
	// Initialize Return Status
	//
	bStatus = false;

	//
	// Configure IO's for LED, User Switch, QSSI and EPI
	//
	ConfigureLEDIOs();
	ConfigureUserSwitchIOs();
	ConfigureEPIIOs();

	//
	// Check if EPI FLASH and SRAM is available. If not,
	// then hang in the loop with D1 Blining if SRAM Failed
	// and D1-D2 Blinking if Flash Failed
	//
	ui32Status = ConfigureEPIInterface();

	if(ui32Status == 0x2)
	{
		while(1)
		{
			Delay(16000000/9);
			HWREG(GPIO_PORTN_BASE + ((LED_DIAG0_PINS | LED_DIAG1_PINS) << 2))
									^= (LED_DIAG1_PINS);
		}
	}
	if(ui32Status == 0x1)
	{
		while(1)
		{
			Delay(16000000/9);
			HWREG(GPIO_PORTN_BASE + ((LED_DIAG0_PINS | LED_DIAG1_PINS) << 2))
									^= (LED_DIAG0_PINS | LED_DIAG1_PINS);
		}
	}

	//
	// Read the User Switch SW1. If the Switch is Low exit the function
	// to go to the Serial Boot Loader to download a firmware. Before
	// doing loader erase the Firmware Entry Table
	// If Switch is high then go and execute the Flash image pointed by
	// the control word
	//
	if(ReadUserSwitch(USER_SW1_PINS))
	{
		bStatus = GetEPIFirmWareImagePointer(&ui16StartSectorNumber, &ui32ImageSizeInBytes);

		//
		// No Image Found so go to main boot loader to download
		// a new firmware image
		if(!bStatus)
		{
			return;
		}

		//
		// If a valid Pointer is available then execute the image in Flasg
		//
		CallApplication(FLASH_APP_START_ADDRESS + (FLASH_SECTOR_SIZE * ui16StartSectorNumber));
	}
	else
	{
		//
		// Erase the Sector for corresponding Fimrware Entry
		// Page Table
		//
		EPIEraseFunction(0);

		return;
	}

	//
	// Execution Should Not Come here. Alternating D1-D2 blinky
	//
	while(1)
	{
		Delay(16000000/9);
		HWREG(GPIO_PORTN_BASE + ((LED_DIAG0_PINS | LED_DIAG1_PINS) << 2))
								= (LED_DIAG0_PINS);
		Delay(16000000/9);
		HWREG(GPIO_PORTN_BASE + ((LED_DIAG0_PINS | LED_DIAG1_PINS) << 2))
								^= (LED_DIAG1_PINS | LED_DIAG0_PINS);
	}

}

//*****************************************************************************
//
//! Sets the Marker for Valid Image
//!
//! \return 1.
//
//*****************************************************************************
void EPIFlashEnd(void)
{
	uint8_t ui8ImagePointer[8];
	uint32_t ui32PointerAddress;

	//
	// Get the address of the Image Pointer Table
	//
	ui32PointerAddress = (FLASH_SECTOR_SIZE * 0);

	//
	// Read 2 Words of Pointer Header to locate the image
	//
	EPIReadFunction(ui32PointerAddress, &ui8ImagePointer[0], 8);

	//
	// Change Marker to 0xFC
	//
	ui8ImagePointer[0] = 0xFC;

	//
	// Program the Sector Address and Size along with the
	// marker for copy
	//
	EPIProgramFunction(ui32PointerAddress, &ui8ImagePointer[0], 8);
}

//*****************************************************************************
//
//! Programs the Address and Size of the Image to be downloaded
//! and the marker indicating a copy is being done.
//!
//! \return 1.
//
//*****************************************************************************
uint32_t
EPIFlashAddressCheck(uint32_t ulAddr, uint32_t ulSize)
{
	uint8_t ui8ImagePointer[8];
	uint32_t ui32PointerAddress;
	uint32_t ui32SectorAddress;

	//
	// Format the data
	//
	ui32SectorAddress  = (ulAddr & ((uint32_t) ~(0xFFFF))) >> 16;
	ui8ImagePointer[0] = 0xFE;
	ui8ImagePointer[1] = 0xFF;
	ui8ImagePointer[2] = (ui32SectorAddress & 0xFF);
	ui8ImagePointer[3] = ((ui32SectorAddress & 0xFF00) >> 8);
	ui8ImagePointer[4] = ((ulSize & 0x000000FF) >> 0);
	ui8ImagePointer[5] = ((ulSize & 0x0000FF00) >> 8);
	ui8ImagePointer[6] = ((ulSize & 0x00FF0000) >> 16);
	ui8ImagePointer[7] = ((ulSize & 0xFF000000) >> 24);


	//
	// Get the address of the Image Pointer Table
	//
	ui32PointerAddress = (FLASH_SECTOR_SIZE * 0);

	//
	// Program the Sector Address and Size along with the
	// marker for copy
	//
	EPIProgramFunction(ui32PointerAddress, &ui8ImagePointer[0], 8);

	//
	// Return Success
	//
	return 1;
}

//*****************************************************************************
//
//! Clears Error Flags
//!
//! \return 1.
//
//*****************************************************************************
void
EPIFlashClearError(void)
{

}

//*****************************************************************************
//
//! Checks if there is any error flag set
//!
//! \return 0.
//
//*****************************************************************************
uint32_t
EPIFlashError(void)
{
	return 0;
}

//*****************************************************************************
//
//! Returns the size of the Serial Flash
//!
//! \return 0xFFFFFFFF
//
//*****************************************************************************
uint32_t
EPIFlashSize(void)
{
	return (0xFFFFFFFF);
}

//*****************************************************************************
//
//! Erases the 64KB Sector in EPI Parallel Flash
//!
//! \return none.
//
//*****************************************************************************
void
EPIFlashErase(uint32_t ulBlockAddr)
{
	//
	// Erase the Sector for corresponding memory pages
	//
	EPIEraseFunction((ulBlockAddr & ((uint32_t) ~(0xFFFF))) >> 16);
}

//*****************************************************************************
//
//! Programs the Image to QSPI Serial Flash
//!
//! \return none.
//
//*****************************************************************************
void EPIFlashProgram(uint32_t ulDstAddr, uint8_t *pucSrcData, uint32_t ulLength)
{
	//
	// Program the Bytes as given by the Array and Lenght at address
	// specified (address is 24-bit)
	//
	EPIProgramFunction(ulDstAddr, pucSrcData, ulLength);
}
