//*****************************************************************************
//
// ektm4c129_epiflash_example.c - Example demonstrating how to configure the
//           EPI bus in HB16 mode for Flash Memory Access
//
// Copyright (c) 2014 Texas Instruments Incorporated.  All rights reserved.
// Software License Agreement
//
//   Redistribution and use in source and binary forms, with or without
//   modification, are permitted provided that the following conditions
//   are met:
//
//   Redistributions of source code must retain the above copyright
//   notice, this list of conditions and the following disclaimer.
//
//   Redistributions in binary form must reproduce the above copyright
//   notice, this list of conditions and the following disclaimer in the
//   documentation and/or other materials provided with the
//   distribution.
//
//   Neither the name of Texas Instruments Incorporated nor the names of
//   its contributors may be used to endorse or promote products derived
//   from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// This is part of revision 2.1.0.12573 of the Tiva Firmware Development Package.
//
//*****************************************************************************

#include <stdbool.h>
#include <stdint.h>
#include "inc/hw_epi.h"
#include "inc/hw_memmap.h"
#include "inc/hw_types.h"
#include "inc/hw_gpio.h"
#include "driverlib/epi.h"
#include "driverlib/gpio.h"
#include "driverlib/pin_map.h"
#include "driverlib/sysctl.h"
#include "driverlib/systick.h"
#include "driverlib/uart.h"
#include "utils/uartstdio.h"

//*****************************************************************************
//
//! \addtogroup epi_examples_list
//! <h1>EPI HB16 Mode (ektm4c129_epiflash_example)</h1>
//!
//! This example shows how to configure the TM4C129 EPI bus in HB16 mode.  It
//! assumes that a Flash memory is attached in HB16 mode (8Mbit to 1Gbit) is
//! attached to EPI0.
//!
//! For the EPI HB16 mode, the pinout is as follows:
//!     Address15:0  - EPI0S15:0
//!     Address25:16 - EPI0S25:16
//!     Data15:0     - EPI0S15:0
//!     /CS0         - EPI0S26
//!     /CS1         - EPI0S27
//!     /OE          - EPI0S28
//!     /WE          - EPI0S29
//!     ALE          - EPI0S30
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - EPI0 peripheral
//! - GPIO Port A peripheral (for EPI0 pins)
//! - GPIO Port B peripheral (for EPI0 pins)
//! - GPIO Port C peripheral (for EPI0 pins)
//! - GPIO Port G peripheral (for EPI0 pins)
//! - GPIO Port H peripheral (for EPI0 pins)
//! - GPIO Port K peripheral (for EPI0 pins)
//! - GPIO Port L peripheral (for EPI0 pins)
//! - GPIO Port M peripheral (for EPI0 pins)
//! - GPIO Port N peripheral (for EPI0 pins)
//! - GPIO Port P peripheral (for EPI0 pins)
//! - GPIO Port Q peripheral (for EPI0 pins)
//! - EPI0S0  - PH0
//! - EPI0S1  - PH1
//! - EPI0S2  - PH2
//! - EPI0S3  - PH3
//! - EPI0S4  - PC7
//! - EPI0S5  - PC6
//! - EPI0S6  - PC5
//! - EPI0S7  - PC4
//! - EPI0S8  - PA6
//! - EPI0S9  - PA7
//! - EPI0S10 - PG1
//! - EPI0S11 - PG0
//! - EPI0S12 - PM3
//! - EPI0S13 - PM2
//! - EPI0S14 - PM1
//! - EPI0S15 - PM0
//! - EPI0S16 - PL0
//! - EPI0S17 - PL1
//! - EPI0S18 - PL2
//! - EPI0S19 - PL3
//! - EPI0S20 - PQ0
//! - EPI0S21 - PQ1
//! - EPI0S22 - PQ2
//! - EPI0S23 - PQ3
//! - EPI0S24 - PK7
//! - EPI0S25 - PK6
//! - EPI0S26 - PL4
//! - EPI0S27 - PB2
//! - EPI0S28 - PB3
//! - EPI0S29 - PP2
//! - EPI0S30 - PP3
//! - EPI0S31 - PK5
//! - EPI0S32 - PK4
//! - EPI0S33 - PL5
//! - EPI0S34 - PN4
//! - EPI0S35 - PN5
//!
//! The following UART signals are configured only for displaying console
//! messages for this example.  These are not required for operation of EPI0.
//! - UART0 peripheral
//! - GPIO Port A peripheral (for UART0 pins)
//! - UART0RX - PA0
//! - UART0TX - PA1
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - None.
//
//*****************************************************************************

//*****************************************************************************
//
// Use the following to specify the GPIO pins used by the HB16 EPI bus.
//
//*****************************************************************************
#define EPI_PORTA_PINS (GPIO_PIN_7 | GPIO_PIN_6)
#define EPI_PORTB_PINS (GPIO_PIN_3 | GPIO_PIN_2)
#define EPI_PORTC_PINS (GPIO_PIN_7 | GPIO_PIN_6 | GPIO_PIN_5 | GPIO_PIN_4)
#define EPI_PORTG_PINS (GPIO_PIN_1 | GPIO_PIN_0)
#define EPI_PORTH_PINS (GPIO_PIN_3 | GPIO_PIN_2 | GPIO_PIN_1 | GPIO_PIN_0)
#define EPI_PORTK_PINS (GPIO_PIN_7 | GPIO_PIN_6 | GPIO_PIN_5 | GPIO_PIN_4)
#define EPI_PORTL_PINS (GPIO_PIN_5 | GPIO_PIN_4 | GPIO_PIN_3 | GPIO_PIN_2 | GPIO_PIN_1 | GPIO_PIN_0)
#define EPI_PORTM_PINS (GPIO_PIN_3 | GPIO_PIN_2 | GPIO_PIN_1 | GPIO_PIN_0)
#define EPI_PORTN_PINS (GPIO_PIN_5 | GPIO_PIN_4)
#define EPI_PORTP_PINS (GPIO_PIN_3 | GPIO_PIN_2)
#define EPI_PORTQ_PINS (GPIO_PIN_3 | GPIO_PIN_2 | GPIO_PIN_1 | GPIO_PIN_0)

//*****************************************************************************
//
// The starting and ending address for the 1Gbit FLASH chip (64Meg x 16bits) on
// the FLASH daughter board.
//
//*****************************************************************************
#define FLASH16_START_ADDRESS  0x60000000

//*****************************************************************************
//
// Use the following to specify the Flash Memory Size, therby fixing the sector
// size, End address and Timing Parameter for Configuration based on Access
// time of the Flash which can be 70ns to 120ns.
// The valid options are 8, 16, 32, 64, 128, 256, 512, 1024
//
//*****************************************************************************
#define FLASH16_VENDOR_ID      0xC2
#define FLASH16_MEMSIZE_MBIT   1024

#if FLASH16_MEMSIZE_MBIT == 8
#define FLASH16_DEVICE_ID      0x220A
#define FLASH16_SECTOR_ADDRESS 0x00010000
#define FLASH16_END_ADDRESS    0x000FFFFF
#elif FLASH16_MEMSIZE_MBIT == 16
#define FLASH16_DEVICE_ID      0x22C4
#define FLASH16_SECTOR_ADDRESS 0x00010000
#define FLASH16_END_ADDRESS    0x001FFFFF
#elif FLASH16_MEMSIZE_MBIT == 32
#define FLASH16_DEVICE_ID      0x22A7
#define FLASH16_SECTOR_ADDRESS 0x00010000
#define FLASH16_END_ADDRESS    0x003FFFFF
#elif FLASH16_MEMSIZE_MBIT == 64
#define FLASH16_DEVICE_ID      0x22C9
#define FLASH16_SECTOR_ADDRESS 0x00010000
#define FLASH16_END_ADDRESS    0x007FFFFF
#elif FLASH16_MEMSIZE_MBIT == 128
#define FLASH16_DEVICE_ID      0x2221
#define FLASH16_SECTOR_ADDRESS 0x00020000
#define FLASH16_END_ADDRESS    0x00FFFFFF
#elif FLASH16_MEMSIZE_MBIT == 256
#define FLASH16_DEVICE_ID      0x2222
#define FLASH16_SECTOR_ADDRESS 0x00020000
#define FLASH16_END_ADDRESS    0x01FFFFFF
#elif FLASH16_MEMSIZE_MBIT == 512
#define FLASH16_DEVICE_ID      0x2223
#define FLASH16_SECTOR_ADDRESS 0x00020000
#define FLASH16_END_ADDRESS    0x03FFFFFF
#elif FLASH16_MEMSIZE_MBIT == 1024
#define FLASH16_DEVICE_ID      0x2228
#define FLASH16_SECTOR_ADDRESS 0x00020000
#define FLASH16_END_ADDRESS    0x07FFFFFF
#else
#error ERROR: Unknown FLASH16_MEMSIZE_MBIT value specified!
#endif

//*****************************************************************************
//
// The Max Count value for the SysTick Timer.
//
//*****************************************************************************
#define SYSTICK_MAX_COUNT	16777216

//*****************************************************************************
//
// Define the Number of Flash locations and create an array to write and read
//
//*****************************************************************************
#define NO_OF_FLASH_LOC	  32

static volatile uint8_t  *g_pui8EPIFlash;
static volatile uint16_t *g_pui16EPIFlash;
static volatile uint32_t *g_pui32EPIFlash;

volatile uint8_t  g_ui8InternalRamArray[NO_OF_FLASH_LOC*2];
volatile uint16_t g_ui16InternalRamArray[NO_OF_FLASH_LOC];
volatile uint32_t g_ui32InternalRamArray[NO_OF_FLASH_LOC/2];

#define BIT16 1
#define BIT8  1

//*****************************************************************************
//
// This function sets up UART0 to be used for a console to display information
// as the example is running.
//
//*****************************************************************************
void
InitConsole(void)
{
    //
    // Enable GPIO port A which is used for UART0 pins.
    // TODO: change this to whichever GPIO port you are using.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);

    //
    // Configure the pin muxing for UART0 functions on port A0 and A1.
    // This step is not necessary if your part does not support pin muxing.
    // TODO: change this to select the port/pin you are using.
    //
    GPIOPinConfigure(GPIO_PA0_U0RX);
    GPIOPinConfigure(GPIO_PA1_U0TX);

    //
    // Enable UART0 so that we can configure the clock.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);

    //
    // Use the internal 16MHz oscillator as the UART clock source.
    //
    UARTClockSourceSet(UART0_BASE, UART_CLOCK_PIOSC);

    //
    // Select the alternate (UART) function for these pins.
    // TODO: change this to select the port/pin you are using.
    //
    GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);

    //
    // Initialize the UART for console I/O.
    //
    UARTStdioConfig(0, 115200, 16000000);
}

//*****************************************************************************
//
// Configure EPI0 in HB16 mode.  The EPI memory space is setup using an a
// simple C array.  This example shows how to read and write to an Flash-16 card
// using the EPI bus in HB16 mode.
//
//*****************************************************************************
int
main(void)
{
    uint32_t ui32ReadData;
    uint32_t ui32Loop;
    uint32_t ui32NoOfHalfWords = NO_OF_FLASH_LOC;
    uint32_t ui32StartTime;
    uint32_t ui32EndTime;
    uint32_t ui32SysClock;
    uint32_t ui32SysClockinMHz;

    #if (FLASH16_MEMSIZE_MBIT == 8 || FLASH16_MEMSIZE_MBIT == 16 \
		|| FLASH16_MEMSIZE_MBIT == 32 || FLASH16_MEMSIZE_MBIT == 64)
	uint32_t ui32ReadID[2];
#endif
#if (FLASH16_MEMSIZE_MBIT == 128 || FLASH16_MEMSIZE_MBIT == 256 || \
    FLASH16_MEMSIZE_MBIT == 512 || FLASH16_MEMSIZE_MBIT == 1024)
	uint32_t ui32ReadID[4];
#endif

    //
    // Set the clocking to run at 120MHz from the PLL.
    //
    ui32SysClock = SysCtlClockFreqSet((SYSCTL_OSC_MAIN | SYSCTL_USE_PLL | SYSCTL_XTAL_25MHZ |
                                       SYSCTL_CFG_VCO_480), 120000000);

    //
    // Compute the System Frequency with the MHz field
    // adjusted for throughput calculation
    //
    ui32SysClockinMHz = ui32SysClock/1000000;

    //
    // Set up the serial console to use for displaying messages.  This is
    // just for this example program and is not needed for EPI operation.
    //
    InitConsole();

    //
    // Display the setup on the console.
    //
    UARTprintf("EPI HB16 FLASH Mode ->\n");
    UARTprintf("  Type: FLASH-16\n");
    UARTprintf("  Starting Address: 0x%08x\n", FLASH16_START_ADDRESS);
    UARTprintf("  End Address: 0x%08x\n",
               (FLASH16_START_ADDRESS + FLASH16_END_ADDRESS));
    UARTprintf("  Data: 16-bit\n");

    //
    // The EPI0 peripheral must be enabled for use.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_EPI0);

    //
    // For this example EPI0 is used with multiple pins on PortA, B, C, G, H,
    // K, L, M and N.  The actual port and pins used may be different on your
    // part, consult the data sheet for more information.
    // TODO: Update based upon the EPI pin assignment on your target part.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOB);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOC);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOG);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOH);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOK);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOL);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOM);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPION);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOP);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOQ);

    //
    // This step configures the internal pin muxes to set the EPI pins for use
    // with EPI.  Please refer to the datasheet for more information about pin
    // muxing.
    //
    GPIOPinConfigure(GPIO_PH0_EPI0S0);
    GPIOPinConfigure(GPIO_PH1_EPI0S1);
    GPIOPinConfigure(GPIO_PH2_EPI0S2);
    GPIOPinConfigure(GPIO_PH3_EPI0S3);
    GPIOPinConfigure(GPIO_PC7_EPI0S4);
    GPIOPinConfigure(GPIO_PC6_EPI0S5);
    GPIOPinConfigure(GPIO_PC5_EPI0S6);
    GPIOPinConfigure(GPIO_PC4_EPI0S7);
    GPIOPinConfigure(GPIO_PA6_EPI0S8);
    GPIOPinConfigure(GPIO_PA7_EPI0S9);
    GPIOPinConfigure(GPIO_PG1_EPI0S10);
    GPIOPinConfigure(GPIO_PG0_EPI0S11);
    GPIOPinConfigure(GPIO_PM3_EPI0S12);
    GPIOPinConfigure(GPIO_PM2_EPI0S13);
    GPIOPinConfigure(GPIO_PM1_EPI0S14);
    GPIOPinConfigure(GPIO_PM0_EPI0S15);
    GPIOPinConfigure(GPIO_PL0_EPI0S16);
    GPIOPinConfigure(GPIO_PL1_EPI0S17);
    GPIOPinConfigure(GPIO_PL2_EPI0S18);
    GPIOPinConfigure(GPIO_PL3_EPI0S19);
    GPIOPinConfigure(GPIO_PQ0_EPI0S20);
    GPIOPinConfigure(GPIO_PQ1_EPI0S21);
    GPIOPinConfigure(GPIO_PQ2_EPI0S22);
    GPIOPinConfigure(GPIO_PQ3_EPI0S23);
    GPIOPinConfigure(GPIO_PK7_EPI0S24);
    GPIOPinConfigure(GPIO_PK6_EPI0S25);
    GPIOPinConfigure(GPIO_PL4_EPI0S26);
    GPIOPinConfigure(GPIO_PB2_EPI0S27);
    GPIOPinConfigure(GPIO_PB3_EPI0S28);
    GPIOPinConfigure(GPIO_PP2_EPI0S29);
    GPIOPinConfigure(GPIO_PP3_EPI0S30);
    GPIOPinConfigure(GPIO_PK5_EPI0S31);
    GPIOPinConfigure(GPIO_PK4_EPI0S32);
    GPIOPinConfigure(GPIO_PL5_EPI0S33);
    GPIOPinConfigure(GPIO_PN4_EPI0S34);
    GPIOPinConfigure(GPIO_PN5_EPI0S35);

    //
    // Configure the GPIO pins for EPI mode.  All the EPI pins require 8mA
    // drive strength in push-pull operation.  This step also gives control of
    // pins to the EPI module.
    //
    GPIOPinTypeEPI(GPIO_PORTA_BASE, EPI_PORTA_PINS);
    GPIOPinTypeEPI(GPIO_PORTB_BASE, EPI_PORTB_PINS);
    GPIOPinTypeEPI(GPIO_PORTC_BASE, EPI_PORTC_PINS);
    GPIOPinTypeEPI(GPIO_PORTG_BASE, EPI_PORTG_PINS);
    GPIOPinTypeEPI(GPIO_PORTH_BASE, EPI_PORTH_PINS);
    GPIOPinTypeEPI(GPIO_PORTK_BASE, EPI_PORTK_PINS);
    GPIOPinTypeEPI(GPIO_PORTL_BASE, EPI_PORTL_PINS);
    GPIOPinTypeEPI(GPIO_PORTM_BASE, EPI_PORTM_PINS);
    GPIOPinTypeEPI(GPIO_PORTN_BASE, EPI_PORTN_PINS);
    GPIOPinTypeEPI(GPIO_PORTP_BASE, EPI_PORTP_PINS);
    GPIOPinTypeEPI(GPIO_PORTQ_BASE, EPI_PORTQ_PINS);

    HWREG(GPIO_PORTA_BASE+GPIO_O_DR8R) |= EPI_PORTA_PINS;
    HWREG(GPIO_PORTB_BASE+GPIO_O_DR8R) |= EPI_PORTB_PINS;
    HWREG(GPIO_PORTC_BASE+GPIO_O_DR8R) |= EPI_PORTC_PINS;
    HWREG(GPIO_PORTG_BASE+GPIO_O_DR8R) |= EPI_PORTG_PINS;
    HWREG(GPIO_PORTH_BASE+GPIO_O_DR8R) |= EPI_PORTH_PINS;
    HWREG(GPIO_PORTK_BASE+GPIO_O_DR8R) |= EPI_PORTK_PINS;
    HWREG(GPIO_PORTL_BASE+GPIO_O_DR8R) |= EPI_PORTL_PINS;
    HWREG(GPIO_PORTM_BASE+GPIO_O_DR8R) |= EPI_PORTM_PINS;
    HWREG(GPIO_PORTN_BASE+GPIO_O_DR8R) |= EPI_PORTN_PINS;
    HWREG(GPIO_PORTP_BASE+GPIO_O_DR8R) |= EPI_PORTP_PINS;
    HWREG(GPIO_PORTQ_BASE+GPIO_O_DR8R) |= EPI_PORTQ_PINS;

    //
    // Sets the usage mode of the EPI module.  For this example we will use
    // the FLASH-16 mode to talk to the external 1Gbit Flash-16 daughter card.
    //
    EPIModeSet(EPI0_BASE, EPI_MODE_HB16);

    EPIDividerSet(EPI0_BASE, 0x1);

#if (FLASH16_MEMSIZE_MBIT == 8 || FLASH16_MEMSIZE_MBIT == 16 \
		|| FLASH16_MEMSIZE_MBIT == 32 || FLASH16_MEMSIZE_MBIT == 64)
    EPIConfigHB16CSSet(EPI0_BASE, 0, (EPI_HB16_CSCFG_ALE_SINGLE_CS | EPI_HB16_ALE_HIGH | EPI_HB16_WRWAIT_0 |
    		EPI_HB16_RDWAIT_0 | EPI_HB16_CSCFG_ALE));

    EPIConfigHB16TimingSet(EPI0_BASE, 0, (EPI_HB16_WRWAIT_MINUS_DISABLE | EPI_HB16_RDWAIT_MINUS_DISABLE));
#endif
#if (FLASH16_MEMSIZE_MBIT == 128 || FLASH16_MEMSIZE_MBIT == 256 || \
    FLASH16_MEMSIZE_MBIT == 512 || FLASH16_MEMSIZE_MBIT == 1024)
    EPIConfigHB16CSSet(EPI0_BASE, 0, (EPI_HB16_CSCFG_ALE_SINGLE_CS | EPI_HB16_ALE_HIGH | EPI_HB16_WRWAIT_1 |
    		EPI_HB16_RDWAIT_1 | EPI_HB16_CSCFG_ALE | EPI_HB16_BSEL));

    EPIConfigHB16TimingSet(EPI0_BASE, 0, (EPI_HB16_WRWAIT_MINUS_DISABLE | EPI_HB16_RDWAIT_MINUS_DISABLE));
#endif

    EPIAddressMapSet(EPI0_BASE, (EPI_ADDR_PER_SIZE_256MB | EPI_ADDR_PER_BASE_A
    							| EPI_ADDR_RAM_SIZE_256MB | EPI_ADDR_RAM_BASE_6) );

    //
    // Set the EPI memory pointer to the base of EPI memory space.  Note that
    // g_pui16EPISRAM is declared as volatile so the compiler should not
    // optimize reads out of the memory.  With this pointer, the memory space
    // is accessed like a simple array.
    //
    g_pui8EPIFlash  = (uint8_t *)(FLASH16_START_ADDRESS + FLASH16_SECTOR_ADDRESS);
    g_pui16EPIFlash = (uint16_t *)(FLASH16_START_ADDRESS + FLASH16_SECTOR_ADDRESS);
    g_pui32EPIFlash = (uint32_t *)(FLASH16_START_ADDRESS + FLASH16_SECTOR_ADDRESS);

    //
    // Silicon and Device ID Check
    //
    HWREGH(FLASH16_START_ADDRESS + 0xAAA) = 0x00AA;
    HWREGH(FLASH16_START_ADDRESS + 0x554) = 0x0055;
    HWREGH(FLASH16_START_ADDRESS + 0xAAA) = 0x0090;
    SysCtlDelay(10);

    ui32ReadID[0] = HWREGH(FLASH16_START_ADDRESS + 0x0);
    ui32ReadID[1] = HWREGH(FLASH16_START_ADDRESS + 0x2);
#if (FLASH16_MEMSIZE_MBIT == 128 || FLASH16_MEMSIZE_MBIT == 256 || \
    FLASH16_MEMSIZE_MBIT == 512 || FLASH16_MEMSIZE_MBIT == 1024)
    ui32ReadID[2] = HWREGH(FLASH16_START_ADDRESS + 0x1C);
    ui32ReadID[3] = HWREGH(FLASH16_START_ADDRESS + 0x1E);
#endif

    //
    // Send a Reset Command to the FLASH to revert it from
    // CFI mode to normal mode
    //
    HWREGH(FLASH16_START_ADDRESS + 0x0) = 0x00F0;

    //
    // If the Flash vendor Check fails then do not execute any further
    //
    if(ui32ReadID[0] != FLASH16_VENDOR_ID)
    {
        UARTprintf("Read of MACRONIX Flash Failed!!!\n");
        UARTprintf("MANUFACTURER ID > %x\n",ui32ReadID[0]);
        UARTprintf("SILICON ID      > %x\n",ui32ReadID[1]);
#if (FLASH16_MEMSIZE_MBIT == 128 || FLASH16_MEMSIZE_MBIT == 256 || \
    FLASH16_MEMSIZE_MBIT == 512 || FLASH16_MEMSIZE_MBIT == 1024)
        UARTprintf("SILICON ID      > %x\n",ui32ReadID[2]);
#endif
        while(1);
    }
    else
    {
    	//
    	// For 8-64 Mbit Flash Silicon ID is the 2nd Word
    	// For 128-1024 Mbit Flash Silicon ID is the 3rd Word
    	//
#if (FLASH16_MEMSIZE_MBIT == 8 || FLASH16_MEMSIZE_MBIT == 16 \
		|| FLASH16_MEMSIZE_MBIT == 32 || FLASH16_MEMSIZE_MBIT == 64)
    	if(ui32ReadID[1] != FLASH16_DEVICE_ID)
    	{
            UARTprintf("Read of SILICON ID Failed > %x\n",ui32ReadID[1]);
    	}
#endif
#if (FLASH16_MEMSIZE_MBIT == 128 || FLASH16_MEMSIZE_MBIT == 256 || \
    FLASH16_MEMSIZE_MBIT == 512 || FLASH16_MEMSIZE_MBIT == 1024)
    	if(ui32ReadID[2] != FLASH16_DEVICE_ID)
    	{
            UARTprintf("Read of SILICON ID Failed > %x\n",ui32ReadID[2]);
    	}
#endif
    }

    //
    // Display the Manufacturer and Silicon ID
    //
    UARTprintf("MANUFACTURER ID > %x\n",ui32ReadID[0]);
    UARTprintf("SILICON ID      > %x\n",ui32ReadID[1]);
#if (FLASH16_MEMSIZE_MBIT == 128 || FLASH16_MEMSIZE_MBIT == 256 || \
    FLASH16_MEMSIZE_MBIT == 512 || FLASH16_MEMSIZE_MBIT == 1024)
    UARTprintf("SILICON ID      > %x\n",ui32ReadID[2]);
#endif


    //
    // Perform Erase of the Sector to make sure that subsequent
    // program can be done
    //
    UARTprintf("FLASH: Sector Erase...\n");

	HWREGB(FLASH16_START_ADDRESS + 0xAAA) = 0x00AA;
    HWREGB(FLASH16_START_ADDRESS + 0x554) = 0x0055;
    HWREGB(FLASH16_START_ADDRESS + 0xAAA) = 0x0080;
    HWREGB(FLASH16_START_ADDRESS + 0xAAA) = 0x00AA;
    HWREGB(FLASH16_START_ADDRESS + 0x554) = 0x0055;

    HWREGB(FLASH16_START_ADDRESS + FLASH16_SECTOR_ADDRESS) = 0x0030;
    ui32ReadData = 0;
    while((ui32ReadData & 0xFF) != 0xFF)
    {
      ui32ReadData = HWREGB(FLASH16_START_ADDRESS + FLASH16_SECTOR_ADDRESS);
    }

    //
    // Perform Read of the Flash Sector to ensure that the locations
    // to be programmed are actually Erased.
    //
   	UARTprintf("FLASH: Erase Check\n");

    for(ui32Loop=0 ; ui32Loop<ui32NoOfHalfWords ; ui32Loop++)
    {
    	if(HWREGH(FLASH16_START_ADDRESS + FLASH16_SECTOR_ADDRESS + ui32Loop*0x2) != 0xffff)
    	{
        	UARTprintf("ERASE FAIL Value of %x is %x\n", (FLASH16_START_ADDRESS + FLASH16_SECTOR_ADDRESS + ui32Loop*0x2),
        			HWREGH(FLASH16_START_ADDRESS + FLASH16_SECTOR_ADDRESS + ui32Loop*0x2));
    	}
    }

    //
    // Now Run Flash Program. For 8-64 Mbit Flash only word program
    // exists. For 128-1024 Mbit Flash Word Buffer Program exists
    //
   	UARTprintf("FLASH: Program\n");
#if (FLASH16_MEMSIZE_MBIT == 8 || FLASH16_MEMSIZE_MBIT == 16 \
		|| FLASH16_MEMSIZE_MBIT == 32 || FLASH16_MEMSIZE_MBIT == 64)
	for(ui32Loop=0 ; ui32Loop<ui32NoOfHalfWords ; ui32Loop++)
    {
    	HWREGH(FLASH16_START_ADDRESS + 0xAAA) 				= 0x00AA;
    	HWREGH(FLASH16_START_ADDRESS + 0x554) 				= 0x0055;
    	HWREGH(FLASH16_START_ADDRESS + 0xAAA) 				= 0x00A0;
    	HWREGH(FLASH16_START_ADDRESS+FLASH16_SECTOR_ADDRESS+(ui32Loop*2)) = (ui32Loop+1);
    	SysCtlDelay(20000);
    }
#endif
#if (FLASH16_MEMSIZE_MBIT == 128 || FLASH16_MEMSIZE_MBIT == 256 || \
    FLASH16_MEMSIZE_MBIT == 512 || FLASH16_MEMSIZE_MBIT == 1024)
   	HWREGH(FLASH16_START_ADDRESS + 0xAAA) = 0x00AA;
    HWREGH(FLASH16_START_ADDRESS + 0x554) = 0x0055;
    HWREGH(FLASH16_START_ADDRESS + FLASH16_SECTOR_ADDRESS)   = 0x0025;
    HWREGH(FLASH16_START_ADDRESS + FLASH16_SECTOR_ADDRESS)   = (ui32NoOfHalfWords - 1);
    for(ui32Loop=0 ; ui32Loop<ui32NoOfHalfWords ; ui32Loop++)
    {
    	HWREGH(FLASH16_START_ADDRESS+FLASH16_SECTOR_ADDRESS+(ui32Loop*2)) = (ui32Loop+1);
    }

	SysCtlDelay(10);
    HWREGH(FLASH16_START_ADDRESS + FLASH16_SECTOR_ADDRESS) = 0x0029;
	SysCtlDelay(100000);
#endif

    //
    // Initialize SysTick Timer
    //
    SysTickPeriodSet(SYSTICK_MAX_COUNT);
    SysTickEnable();

    //
    // Clear Internal SRAM
    //
    for(ui32Loop=0;ui32Loop<(NO_OF_FLASH_LOC/2);ui32Loop++)
    {
    	g_ui32InternalRamArray[ui32Loop] = 0x0;
    }

    //
    // Perform Check for 32-bit Write and Read
    // Get the Start Count
    //
    ui32StartTime = SysTickValueGet();

    //
    // Copy from external Flash to internal SRAM
    //
    for(ui32Loop=0;ui32Loop<(NO_OF_FLASH_LOC/2);ui32Loop++)
    {
    	g_ui32InternalRamArray[ui32Loop] = g_pui32EPIFlash[ui32Loop];
    }

    //
    // Get the End Count
    //
    ui32EndTime = SysTickValueGet();

    UARTprintf("32-bit FLASH Read  : %02d.%03d MBps\n",((NO_OF_FLASH_LOC*2*ui32SysClockinMHz)/(ui32StartTime-ui32EndTime)),
    		((NO_OF_FLASH_LOC*2*ui32SysClockinMHz)%(ui32StartTime-ui32EndTime)));

    //
    // Run Checker to see all data is fine or not
    //
    for(ui32Loop=0;ui32Loop<(NO_OF_FLASH_LOC/2);ui32Loop++)
    {
    	if(g_ui32InternalRamArray[ui32Loop] != HWREG(FLASH16_START_ADDRESS + FLASH16_SECTOR_ADDRESS + ui32Loop*0x4))
    	{
    	    UARTprintf("32-bit Read and/or write failure @ %d location!",ui32Loop);
    	    return(0);
    	}
    }

#if BIT16
    //
    // Clear Internal SRAM
    //
    for(ui32Loop=0;ui32Loop<(NO_OF_FLASH_LOC);ui32Loop++)
    {
    	g_ui16InternalRamArray[ui32Loop] = 0x0;
    }

    //
    // Perform Check for 32-bit Write and Read
    // Get the Start Count
    //
    ui32StartTime = SysTickValueGet();

    //
    // Copy from external Flash to internal SRAM
    //
    for(ui32Loop=0;ui32Loop<(NO_OF_FLASH_LOC);ui32Loop++)
    {
    	g_ui16InternalRamArray[ui32Loop] = g_pui16EPIFlash[ui32Loop];
    }

    //
    // Get the End Count
    //
    ui32EndTime = SysTickValueGet();

    UARTprintf("16-bit FLASH Read  : %02d.%03d MBps\n",((NO_OF_FLASH_LOC*2*ui32SysClockinMHz)/(ui32StartTime-ui32EndTime)),
    		((NO_OF_FLASH_LOC*2*ui32SysClockinMHz)%(ui32StartTime-ui32EndTime)));

    //
    // Run Checker to see all data is fine or not
    //
    for(ui32Loop=0;ui32Loop<(NO_OF_FLASH_LOC);ui32Loop++)
    {
    	if(g_ui16InternalRamArray[ui32Loop] != HWREGH(FLASH16_START_ADDRESS + FLASH16_SECTOR_ADDRESS + ui32Loop*0x2))
    	{
    	    UARTprintf("32-bit Read and/or write failure @ %d location!",ui32Loop);
    	    return(0);
    	}
    }
#endif

#if BIT8
    //
    // Clear Internal SRAM
    //
    for(ui32Loop=0;ui32Loop<(NO_OF_FLASH_LOC*2);ui32Loop++)
    {
    	g_ui8InternalRamArray[ui32Loop] = 0x0;
    }

    //
    // Perform Check for 32-bit Write and Read
    // Get the Start Count
    //
    ui32StartTime = SysTickValueGet();

    //
    // Copy from external Flash to internal SRAM
    //
    for(ui32Loop=0;ui32Loop<(NO_OF_FLASH_LOC*2);ui32Loop++)
    {
    	g_ui8InternalRamArray[ui32Loop] = g_pui8EPIFlash[ui32Loop];
    }

    //
    // Get the End Count
    //
    ui32EndTime = SysTickValueGet();

    UARTprintf("8-bit  FLASH Read  : %02d.%03d MBps\n",((NO_OF_FLASH_LOC*2*ui32SysClockinMHz)/(ui32StartTime-ui32EndTime)),
    		((NO_OF_FLASH_LOC*2*ui32SysClockinMHz)%(ui32StartTime-ui32EndTime)));

    //
    // Run Checker to see all data is fine or not
    //
    for(ui32Loop=0;ui32Loop<(NO_OF_FLASH_LOC*2);ui32Loop++)
    {
    	if(g_ui8InternalRamArray[ui32Loop] != HWREGB(FLASH16_START_ADDRESS + FLASH16_SECTOR_ADDRESS + ui32Loop))
    	{
    	    UARTprintf("8-bit Read and/or write failure @ %d location!",ui32Loop);
    	    return(0);
    	}
    }
#endif

    //
    // End the Sample Code
    //
    while(1)
    {
    }
}
